<?php

namespace NoccyLabs\SerialPort;



class SerialPort
{

    private $device;

    private $speed = 9600;

    private $bits = 8;

    private $parity = 'n';

    private $stopbits = 1;

    private $raw = true;

    private $fh;

    public function __construct($path, int $speed, int $bits=8, string $parity='n', int $stopbits=1, bool $raw=true)
    {

        // Grab some sensible defaults if one or more values are missing or invalid
        $this->speed = $speed ?: 9600;
        $this->bits = in_array($bits, [7, 8]) ? $bits : 8;
        $this->parity = in_array($parity, ['n', 'e', 'o']) ? $parity : 'n';
        $this->stopbits = in_array($stopbits, [1, 2]) ? $stopbits : 1;
        $this->raw = $raw;

        self::setupSerial($path, $this->speed, $this->bits, $this->parity, $this->stopbits, $this->raw);
        $this->fh = fopen($path, "rb+");
    }

    public static function setupSerial(string $device, int $speed, int $bits, string $parity, int $stopbits, bool $raw)
    {

        $cmdl = "stty -F {$device} {$speed} "
                . ($raw?" raw":"")
                . " -icanon min 0 time 1"
                . " cs8"
                . ($parity=='n'?" -parenb":($parity=="e"?" parenb -parodd":" parenb parodd"))
                . ($stopbits==1?" -cstopb":" cstopb")
                ;

        echo "# {$cmdl}\n";
        exec("exec {$cmdl} < {$device}");

        // exec("stty -F {$device} raw -echo -echoe -echok cs8 -cstopb -parenb");


    }

    function read($count)
    {
        return fread($this->fh, $count);
    }

    function gets($length=null)
    {
        if (!$this->fh) throw new \RuntimeException("fh invalid");
        return fgets($this->fh, $length);
    }

    function write($data)
    {
        if (!$this->fh) throw new \RuntimeException("fh invalid");
        return fwrite($this->fh, $data);
    }

    function eof()
    {
        if (!$this->fh) throw new \RuntimeException("fh invalid");
        return feof($this->fh);
    }

}
