<?php

namespace NoccyLabs\SerialPort;



class SerialProtocolWrapper
{

    private $device;

    private $speed = 9600;

    private $bits = 8;

    private $parity = 'n';

    private $stopbits = 1;

    private $raw = true;

    private $fh;

    function stream_open($path, $mode, $options, &$opened_path)
    {
        // Extract the device name and the modes
        $device = substr($path, strpos($path, "://") + 3);
        [$device, $modes] = explode(",", $device, 2);

        // If the device name doesn't start with a slash, point it to /dev/
        if (substr($device, 0, 1) !== "/") {
            $device = "/dev/".$device;
        }

        // Parse the modes
        $modes = explode(",", $modes);
        // In order;
        //   speed,databits,parity,stopbits
        // Example:
        //   9600,8,n,1
        $speed = array_shift($modes);
        $bits = array_shift($modes);
        $parity = strtolower(array_shift($modes));
        $stopbits = array_shift($modes);

        // Grab some sensible defaults if one or more values are missing or invalid
        $this->speed = $speed ?: 9600;
        $this->bits = in_array($bits, [7, 8]) ? $bits : 8;
        $this->parity = in_array($parity, ['n', 'e', 'o']) ? $parity : 'n';
        $this->stopbits = in_array($stopbits, [1, 2]) ? $stopbits : 1;

        // Parse the flags
        //   +/-raw   Change
        while ($flag = array_shift($modes)) {
            switch ($flag) {
                case '+raw':
                    $this->raw = true;
                    break;
                case '-raw':
                    $this->raw = false;
                    break;
            }
        }

        //fprintf(STDERR, "open: dev=%s, mode=%s, rate=%d, bits=%d, parity=%s, stopbits=%d, raw=%d\n", $device, $mode, $this->speed, $this->bits, $this->parity, $this->stopbits, $this->raw);

        $this->fh = @fopen($device, $mode);
        if (!$this->fh) {
            throw new SerialException(error_get_last()['message']);
        }

        SerialPort::setupSerial($device, $this->speed, $this->bits, $this->parity, $this->stopbits, $this->raw);

        $opened_path = $device;

        return true;
    }

    function stream_read($count)
    {
        return fread($this->fh, $count);
    }

    function stream_write($data)
    {
        echo "> {$data}\n";
        return fwrite($this->fh, $data);
    }

    function stream_eof()
    {
        return feof($this->fh);
    }

    public static function register()
    {
        stream_wrapper_register("serial", SerialProtocolWrapper::class);
    }

}
